/*
 * This file is part of Jenirok.
 *
 * Jenirok is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Jenirok is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Jenirok.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QDebug>
#include <QApplication>
#include <QtCore/QTimerEvent>
#include <QtCore/QVariant>
#include <icd/dbus_api.h>
#include "connectionmanager.h"


ConnectionManager::ConnectionManager(QObject* parent): QObject(parent),
ready_(false), connected_(false), timeout_(false), ignoreStateChanges_(false),
timer_(0)
{
    QDBusConnection systemBus = QDBusConnection::systemBus();

    icd2interface_ = new QDBusInterface(ICD_DBUS_API_INTERFACE,
                                        ICD_DBUS_API_PATH, ICD_DBUS_API_INTERFACE,
                                        systemBus, this);

    systemBus.connect(ICD_DBUS_API_INTERFACE, ICD_DBUS_API_PATH,
                      ICD_DBUS_API_INTERFACE, ICD_DBUS_API_STATE_SIG,
                      this, SLOT(stateChange(const QDBusMessage&)));

    systemBus.connect(ICD_DBUS_API_INTERFACE, ICD_DBUS_API_PATH,
                      ICD_DBUS_API_INTERFACE, ICD_DBUS_API_CONNECT_SIG,
                      this, SLOT(connectionChange(const QDBusMessage&)));


}

bool ConnectionManager::connect()
{
    ready_ = false;
    ignoreStateChanges_ = true;
    unsigned int flags = static_cast<unsigned int>(ICD_CONNECTION_FLAG_USER_EVENT);
    icd2interface_->call(ICD_DBUS_API_CONNECT_REQ, QVariant(flags));
    waitSignal();
    return connected_;
}

bool ConnectionManager::disconnect()
{
    if(!connected_)
    {
        return false;
    }

    ready_ = false;
    ignoreStateChanges_ = false;
    unsigned int flags = static_cast<unsigned int>(ICD_CONNECTION_FLAG_USER_EVENT);
    icd2interface_->call(ICD_DBUS_API_DISCONNECT_REQ, QVariant(flags));
    connected_ = false;
    return true;
}

bool ConnectionManager::isConnected()
{
    ready_ = false;
    ignoreStateChanges_ = false;
    QDBusMessage rep = icd2interface_->call(ICD_DBUS_API_STATE_REQ);

    unsigned int numOfReplies = rep.arguments().value(0).value<unsigned int>();

    if(numOfReplies == 0)
    {
        return false;
    }

    waitSignal();
    return connected_;
}

void ConnectionManager::stateChange(const QDBusMessage& rep)
{
    if(ignoreStateChanges_)
    {
        return;
    }

    unsigned int status = rep.arguments().value(7).value<unsigned int>();

    switch(status)
    {
    case ICD_STATE_CONNECTING:
        qDebug() << "Connecting";
        break;
    case ICD_STATE_CONNECTED:
        connected_ = true;
        ready_ = true;
        qDebug() << "Connected";
        break;
    case ICD_STATE_DISCONNECTING:
        qDebug() << "Disconnecting";
        break;
    case ICD_STATE_DISCONNECTED:
        connected_ = false;
        ready_ = true;
        qDebug() << "Disconnected";
        break;
    case ICD_STATE_LIMITED_CONN_ENABLED:
        connected_ = true;
        ready_ = true;
        qDebug() << "Limited connection enabled";
        break;
    case ICD_STATE_LIMITED_CONN_DISABLED:
        connected_ = false;
        ready_ = true;
        qDebug() << "Limited connection disabled";
        break;
    case ICD_STATE_SEARCH_START:
        qDebug() << "Search start";
        break;
    case ICD_STATE_SEARCH_STOP:
        qDebug() << "Search stop";
        break;
    case ICD_STATE_INTERNAL_ADDRESS_ACQUIRED:
        qDebug() << "Internal address acquired";
        break;
    default:
        qDebug() << "Unknown connection status";
        break;
    }

}

void ConnectionManager::connectionChange(const QDBusMessage& rep)
{
    unsigned int status = rep.arguments().value(6).value<unsigned int>();

    switch(status)
    {
    case ICD_CONNECTION_SUCCESSFUL:
        connected_ = true;
        ready_ = true;
        qDebug() << "Connection successful";
        break;
    case ICD_CONNECTION_NOT_CONNECTED:
        connected_ = false;
        ready_ = true;
        qDebug() << "Connection not connected";
        break;
    case ICD_CONNECTION_DISCONNECTED:
        connected_ = false;
        ready_ = true;
        qDebug() << "Connection disconnected";
        break;
    default:
        qDebug() << "Unknown connection status";
        break;
    }
}

bool ConnectionManager::waitSignal()
{
    timeout_ = false;
    timer_ = startTimer(TIMEOUT);

    while(!ready_ && !timeout_)
    {
        QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents);
    }

    killTimer(timer_);

    return ready_ || !timeout_;
}

void ConnectionManager::timerEvent(QTimerEvent* event)
{
    Q_UNUSED(event);
    killTimer(timer_);
    timeout_ = true;
    timer_ = 0;

    qDebug() << "Connection request timed out";
}
